#pragma warning ( disable : 4786 )

#include "dxstdafx.h"
#include "D3DManager.h"
//#include <dxerr9.h>
/*
_TCHAR * GetError(HRESULT hr)
{
	static _TCHAR buffer[2048];

	_tcscpy(buffer,DXGetErrorString9(hr));

	return buffer;
}

void dxtrace( _TCHAR * fmtStr, ...)
{
	int ret;
	_TCHAR buffer[2048];

	va_list marker;
	va_start( marker,fmtStr);
	ret=_vstprintf( buffer,fmtStr,marker);
	va_end(marker);

	OutputDebugString(buffer);
}
*/

std::map<HWND,CD3DManager*> CD3DManager::ms_mHwnd2PtrMap;

CD3DManager::CD3DManager()
{
	m_pd3dDevice = NULL;
	m_pD3D       = NULL; 

	ZeroMemory( &m_d3ddm, sizeof(m_d3ddm) );
	ZeroMemory( &m_d3dpp, sizeof(m_d3dpp) );
	ZeroMemory( &m_d3dCaps, sizeof(m_d3dCaps) );

	m_dwBehavior = 0x00000000;
	m_dwAdapter  = 0x00000000;

	m_hDevWindow = NULL;
	m_hInstance =  NULL;

	m_bWindowed = TRUE;

	m_bInitialized = FALSE;

	m_uiWidth = 0;
	m_uiHeight = 0;

	m_dTime = 0.0;
	m_dElapsedTime = 0.0;

	m_bHookedWnd = FALSE;
	m_lpfnChildWndProc = NULL;
}

CD3DManager::~CD3DManager()
{
	//Shutdown();
}

HRESULT CD3DManager::Initialize(HWND hWnd, BOOL bRenderOnPaint, BOOL bHookWnd)
{
	if (!m_bInitialized) 
	{
		dxtrace(_T("CD3DManager::Initialize()\n"));

		HRESULT hr = E_FAIL;

		QueryPerformanceFrequency(&m_qwTicksPerSec);
		QueryPerformanceCounter(&m_qwPerfCount);
		
		m_hInstance = GetModuleHandle(NULL);
		m_hDevWindow = hWnd;
		m_bRenderOnPaint = bRenderOnPaint;
		
		RECT rcWindowClient;

		GetClientRect(hWnd,&rcWindowClient);
		m_uiWidth = rcWindowClient.right - rcWindowClient.left;
		m_uiHeight = rcWindowClient.bottom - rcWindowClient.top;

		if(bHookWnd)
			hookWindow(hWnd);
		
		if( NULL == (m_pD3D = Direct3DCreate9( D3D_SDK_VERSION )))
		{	
			dxtrace(TEXT("D3D obj creation failed"));
			return hr;
		}

		ATTEMPT( m_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &m_d3ddm) );
		
		ZeroMemory( &m_d3dpp, sizeof(m_d3dpp) );
		m_d3dpp.Windowed = m_bWindowed;
		m_d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
		m_d3dpp.BackBufferFormat = m_d3ddm.Format;
		m_d3dpp.EnableAutoDepthStencil = TRUE;
		m_d3dpp.AutoDepthStencilFormat = D3DFMT_D24S8;
		m_d3dpp.BackBufferWidth  = m_uiWidth;
		m_d3dpp.BackBufferHeight = m_uiHeight;
		m_d3dpp.hDeviceWindow = m_hDevWindow;
		m_d3dpp.BackBufferCount = 1;
		m_d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

		m_dwBehavior = D3DCREATE_HARDWARE_VERTEXPROCESSING;
		m_dwAdapter =  D3DADAPTER_DEFAULT;

		ATTEMPT( m_pD3D->CreateDevice( m_dwAdapter, D3DDEVTYPE_HAL, m_hDevWindow,
										  m_dwBehavior,&m_d3dpp, &m_pd3dDevice ) );

		// Reject devices that cannot perform 8-bit stencil buffering. 
		// The following example assumes that pCaps is a valid pointer 
		
		m_pd3dDevice->GetDeviceCaps( &m_d3dCaps );

		ATTEMPT( m_pD3D->CheckDeviceFormat( m_d3dCaps.AdapterOrdinal,m_d3dCaps.DeviceType, m_d3ddm.Format,  
										       D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D24S8 ) );


		dxtrace(TEXT("Created device, m_pd3dDevice:%x\n"),m_pd3dDevice);

		ATTEMPT(ApplyDefaultStates());

		ATTEMPT(OnInitDeviceObjects(m_pd3dDevice));
		ATTEMPT(OnRestoreDeviceObjects());

		ATTEMPT(OnInit());
		
		m_bInitialized = TRUE;
	}

	return S_OK;
}

HRESULT CD3DManager::Shutdown()
{
	if(m_bInitialized) 
	{
		HRESULT hr;
		
		dxtrace(_T("CD3DManager::Shutdown()\n"));

		ATTEMPT(OnInvalidateDeviceObjects());

		ATTEMPT(OnDeleteDeviceObjects());

		ATTEMPT(OnShutdown());

		ATTEMPT(m_pd3dDevice->Release());

		ATTEMPT(m_pD3D->Release());

		m_bInitialized = FALSE;
		return S_OK;
	}
	return E_FAIL;
}

HRESULT CD3DManager::Reset()
{
	HRESULT hr;
	dxtrace(_T("Trace:Entered Reset, m_pd3dDevice:%x\n"),m_pd3dDevice);

	ATTEMPT( OnInvalidateDeviceObjects() );
	
	if( FAILED( hr = m_pd3dDevice->Reset(&m_d3dpp) ) )
	{
		ATTEMPT( OnDeleteDeviceObjects() );

		dxtrace(_T("Trace:Reset Failed, hr:%x err:%s\n"),hr,GetError(hr));
		dxtrace(_T("Trace:Attempting device recreation\n"));
		
		ATTEMPT( m_pd3dDevice->Release() );

		ATTEMPT( m_pD3D->CreateDevice( m_dwAdapter, D3DDEVTYPE_HAL, m_d3dpp.hDeviceWindow,
								   m_dwBehavior,
								   &m_d3dpp, &m_pd3dDevice ) );

		dxtrace(_T("Trace:ReCreated device, m_pd3dDevice:%x\n"),m_pd3dDevice);

		ATTEMPT( OnInitDeviceObjects(m_pd3dDevice) );
	}

	
    ATTEMPT( OnRestoreDeviceObjects() );

	dxtrace(_T("Trace:Exited Reset, m_pd3dDevice:%x\n"),m_pd3dDevice);
	return S_OK;
}

void CD3DManager::updateTime()
{
	LARGE_INTEGER qwTemp;

	QueryPerformanceCounter(&qwTemp);

	m_dElapsedTime = (qwTemp.QuadPart - m_qwPerfCount.QuadPart) / 
		((double)m_qwTicksPerSec.QuadPart);

	m_dTime += m_dElapsedTime;
	m_qwPerfCount = qwTemp;
}

HRESULT CD3DManager::ProcessFrame()
{
	HRESULT hr = S_OK;

	updateTime();

	ATTEMPT(OnUpdate(m_dTime,m_dElapsedTime));
	ATTEMPT(OnRender(m_pd3dDevice));

	return hr;
}

HRESULT CD3DManager::Resize()
{
	RECT rcWindowClient;
	GetClientRect(m_hDevWindow, &rcWindowClient);

	m_uiWidth = rcWindowClient.right - rcWindowClient.left;
	m_uiHeight = rcWindowClient.bottom - rcWindowClient.top;

	SetBBDim(m_uiWidth,m_uiHeight);

	return Reset();
}


HRESULT CD3DManager::HandleW32Msg(HWND hWnd, UINT message,WPARAM wparam, LPARAM lparam)
{
	return OnMsg(hWnd,message,wparam,lparam);
}


/////////////////////////////////////////////////////////////////////////////////

HRESULT CD3DManager::ApplyDefaultStates()
{
	HRESULT hr;

	ATTEMPT( m_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE ) );

	return S_OK;
}


HRESULT CD3DManager::SetBBDim(UINT uiWidth, UINT uiHeight, BOOL bEnableAutoDepthStencil, UINT uiBackBufCount)
{
	m_d3dpp.BackBufferWidth        = uiWidth;
    m_d3dpp.BackBufferHeight       = uiHeight;
	m_d3dpp.EnableAutoDepthStencil = bEnableAutoDepthStencil;
	m_d3dpp.BackBufferCount        = uiBackBufCount;

	return S_OK;
}

BOOL CD3DManager::SetDevWindow(HWND hWnd)
{
	m_d3dpp.hDeviceWindow = hWnd;
	return TRUE;
}


BOOL CD3DManager::SetWindowed(BOOL bWindowed)
{
	m_d3dpp.Windowed = m_bWindowed = bWindowed;
	return bWindowed;
}

CD3DManager* CD3DManager::getHWNDPtr(HWND hHwnd)
{
	std::map<HWND,CD3DManager*>::iterator iter;
	iter = ms_mHwnd2PtrMap.find(hHwnd);

	if(iter!=ms_mHwnd2PtrMap.end())
	{
		return((*iter).second);
	}

	return NULL;
}

void CD3DManager::addHWNDPtr(HWND hWnd)
{
	ms_mHwnd2PtrMap[hWnd] = this;	
}

BOOL CD3DManager::hookWindow( HWND hWndNew)
{
	HRESULT hr=S_OK;

	m_bHookedWnd = TRUE;

	addHWNDPtr(m_hDevWindow);

	if(m_lpfnChildWndProc = (WNDPROC)SetWindowLong( m_hDevWindow, GWL_WNDPROC, (LONG)RenderWndProc ) )
		return TRUE;

	return FALSE;
}

void CD3DManager::unhookWindow()
{
	if(m_bHookedWnd)
	{
		m_bHookedWnd= FALSE;
	}
}

LRESULT CALLBACK CD3DManager::RenderWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LRESULT lRet = 0;

	CD3DManager* pThis = getHWNDPtr(hwnd);
	if(pThis==NULL)
		return lRet;

	switch(uMsg)
	{
	case WM_EXITSIZEMOVE:
		pThis->Resize();
		break;

	case WM_PAINT:
		if(pThis->m_bRenderOnPaint)
			pThis->ProcessFrame();
		break;

	case WM_GETMINMAXINFO:
		((LPMINMAXINFO)lParam)->ptMinTrackSize.x=100;
		((LPMINMAXINFO)lParam)->ptMinTrackSize.y=100;
		break;		

	case WM_CLOSE:
		pThis->unhookWindow();
		break;
	}

	pThis->HandleW32Msg(hwnd,uMsg,wParam,lParam);

	lRet = CallWindowProc(pThis->m_lpfnChildWndProc, hwnd, uMsg, wParam,lParam);

	return lRet;

}	
