// DeviceManager.h: interface for the CD3DManager class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#if defined(DEBUG) || defined(_DEBUG)
#ifndef D3D_DEBUG_INFO
#define D3D_DEBUG_INFO
#endif
#endif

#include <vector>
#include <map>
#include <windows.h>
#include <d3d9.h>
#include <tchar.h>

#ifdef _EXPORTING
#define CLASS_DECLSPEC    __declspec(dllexport)
#else
#define CLASS_DECLSPEC    __declspec(dllimport)
#endif

class CLASS_DECLSPEC CD3DManager
{
public:
	CD3DManager();
	virtual ~CD3DManager();

	HRESULT  Initialize(HWND hWnd, BOOL bRenderOnPaint=TRUE, BOOL bHookWnd=FALSE);
	HRESULT  Shutdown();                          
	HRESULT  Reset();
	HRESULT  ProcessFrame();

	HRESULT  Resize();
	HRESULT  HandleW32Msg(HWND hWnd, UINT message,WPARAM wparam, LPARAM lparam);

	LPDIRECT3DDEVICE9 GetDevice(){return m_pd3dDevice;}         
	LPDIRECT3D9       GetD3D(){return m_pD3D;}
	D3DCAPS9          GetDeviceCaps(){return m_d3dCaps;}
	BOOL              GetRenderOnPaint() {return m_bRenderOnPaint;}

	static LRESULT CALLBACK RenderWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

protected:
	HRESULT ApplyDefaultStates();
	HRESULT SetBBDim(UINT uiWidth, UINT uiHeight, BOOL bEnableAutoDepthStencil = TRUE, UINT uiBackBufferCount = 1);
	BOOL SetDevWindow(HWND hWnd);
	BOOL SetWindowed(BOOL bWindowed);
	void updateTime();

	static  CD3DManager* getHWNDPtr(HWND hWnd);
	void addHWNDPtr(HWND hWnd);
	BOOL hookWindow(HWND hWnd);
	void unhookWindow();

	virtual HRESULT OnInit() { return S_OK; }
	virtual HRESULT OnShutdown() { return S_OK; }
	virtual HRESULT OnUpdate(double dTime, double dElapsedTime) { return S_OK; }
	virtual HRESULT OnMsg(HWND hWnd, UINT message,WPARAM wparam, LPARAM lparam) { return S_OK; }
	virtual HRESULT OnRender(LPDIRECT3DDEVICE9 pd3dDevice) { return S_OK; }
	virtual HRESULT OnInitDeviceObjects(LPDIRECT3DDEVICE9 pd3dDevice) { return S_OK; }
	virtual HRESULT OnRestoreDeviceObjects() { return S_OK; }
	virtual HRESULT OnInvalidateDeviceObjects() { return S_OK; }
	virtual HRESULT OnDeleteDeviceObjects() { return S_OK; }

	LPDIRECT3DDEVICE9      m_pd3dDevice;           /// Pointer to the Device interface.
	LPDIRECT3D9            m_pD3D;                 /// Pointer to the D3D object.

	D3DDISPLAYMODE         m_d3ddm;                /// Current display mode.
	D3DCAPS9               m_d3dCaps;              /// The Device Capabilties structure.
	D3DPRESENT_PARAMETERS  m_d3dpp;                /// D3D pHRESULTentation parameters.

	DWORD                  m_dwBehavior;           /// Signifies if the device is multithreaded, hardware vp, etc.
	DWORD				   m_dwAdapter;            /// Which GPU, typically left to default.

	HWND                   m_hDevWindow;           /// A handle to the devices main window
	HINSTANCE              m_hInstance;            /// Application instance handle.

	BOOL				   m_bWindowed;

	BOOL                   m_bInitialized;

	UINT                   m_uiWidth;
	UINT                   m_uiHeight;

	LARGE_INTEGER          m_qwTicksPerSec;
	LARGE_INTEGER          m_qwPerfCount;

	double                 m_dTime;
	double                 m_dElapsedTime;

	BOOL                        m_bRenderOnPaint;

	BOOL                        m_bHookedWnd;
	WNDPROC                     m_lpfnChildWndProc;
	static std::map<HWND,CD3DManager*> ms_mHwnd2PtrMap;
};

/*
#ifndef SAFE_DELETE
#define SAFE_DELETE(p)       { if(p) { delete (p);     (p)=NULL; } }
#endif    
#ifndef SAFE_DELETE_ARRAY
#define SAFE_DELETE_ARRAY(p) { if(p) { delete[] (p);   (p)=NULL; } }
#endif    
#ifndef SAFE_RELEASE
#define SAFE_RELEASE(p)      { if(p) { (p)->Release(); (p)=NULL; } }
#endif

#define ATTEMPT( EXPRESSION )\
	if( FAILED ( hr = EXPRESSION ) ){\
	dxtrace(_T("TraceDW: File:%s Line:%d FAILED: hr:0x%x expr:%s\n"), _T(__FILE__), __LINE__, hr, _T( #EXPRESSION ) );\
	return hr;}\

#define PTR_ATTEMPT( EXPRESSION )\
	if( (EXPRESSION) == NULL ){\
	dxtrace(_T("TraceDW: File:%s Line:%d FAILED: hr:0x%x expr:%s\n"), _T(__FILE__), __LINE__, hr, _T( #EXPRESSION ) );\
	return E_FAIL;}\

*/
