#include "dxstdafx.h"
#include "D3DTestRender.h"


CD3DTestRender::CD3DTestRender()
{
	m_dwMouseButtonStates = 0;
	m_iMouseX = 0;
	m_iMouseY = 0;
}

CD3DTestRender::~CD3DTestRender()
{
}

HRESULT CD3DTestRender::OnInit()
{
	return S_OK;
}

HRESULT CD3DTestRender::OnShutdown()
{
	return S_OK;
}

HRESULT CD3DTestRender::OnMsg(HWND hWnd, UINT message,WPARAM wparam, LPARAM lparam)
{
	if(hWnd == m_hDevWindow)
	{
		switch(message)
		{
		case WM_MOUSEMOVE:
		case WM_LBUTTONDOWN:
		case WM_LBUTTONUP:
		case WM_RBUTTONDOWN:
		case WM_RBUTTONUP:
		case WM_MBUTTONDOWN:
		case WM_MBUTTONUP:
			m_iMouseX = LOWORD(lparam);
			m_iMouseY = HIWORD(lparam);
			if(m_iMouseX >= 8192)
				m_iMouseX = m_iMouseX - 65536;
			if(m_iMouseY >= 8192)
				m_iMouseY = m_iMouseY - 65536;
			m_dwMouseButtonStates = (DWORD)wparam;
		default:
			break;
		}
	}

	return S_OK;
}

HRESULT CD3DTestRender::OnUpdate(double dTime, double dElapsedTime)
{
	m_Time = dTime;
	m_FPS = 1.0 / dElapsedTime;

	return S_OK;
}

HRESULT CD3DTestRender::OnRender(LPDIRECT3DDEVICE9 pd3dDevice)
{

	HRESULT hr;
	// Clear the render target and the zbuffer 
	ATTEMPT(pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xff000000, 1.0f, 0) );

	// Render the scene
	if(SUCCEEDED( pd3dDevice->BeginScene()))
	{
		RECT rc;
		TCHAR szInfo[256];

		_stprintf(szInfo,_T("Time: %f\nFPS: %f\nRes: %d %d\nMState:0x%x\nMXY:%d %d"),
			      m_Time,m_FPS,m_uiWidth,m_uiHeight,m_dwMouseButtonStates,m_iMouseX,m_iMouseY);

		SetRect(&rc, 10, 10, 0, 0 );
		m_pFont->DrawText( NULL, szInfo, -1, &rc, DT_NOCLIP, D3DXCOLOR( 1.0f, 0.0f, 0.0f, 1.0f ));

		ATTEMPT(pd3dDevice->EndScene());

		ATTEMPT(pd3dDevice->Present( NULL, NULL, NULL, NULL ));
	}

	Sleep(10);

	return S_OK;
}

HRESULT CD3DTestRender::OnInitDeviceObjects(LPDIRECT3DDEVICE9 pd3dDevice)
{
	HRESULT hr;

	m_pd3dDevice = pd3dDevice;

	hr = D3DXCreateFont(m_pd3dDevice,            // D3D device
						12,               // Height
						0,                     // Width
						FW_REGULAR,               // Weight
						1,                     // MipLevels, 0 = autogen mipmaps
						FALSE,                 // Italic
						DEFAULT_CHARSET,       // CharSet
						OUT_DEFAULT_PRECIS,    // OutputPrecision
						DEFAULT_QUALITY,       // Quality
						DEFAULT_PITCH | FF_DONTCARE, // PitchAndFamily
						L"Courier",              // pFaceName
						&m_pFont);              // ppFont

	return hr;
}

HRESULT CD3DTestRender::OnRestoreDeviceObjects()
{
	m_pFont->OnResetDevice();

	return S_OK;
}

HRESULT CD3DTestRender::OnInvalidateDeviceObjects()
{
	m_pFont->OnLostDevice();

	return S_OK;
}

HRESULT CD3DTestRender::OnDeleteDeviceObjects()
{
	SAFE_RELEASE(m_pFont);

	return S_OK;
}